@@ -39,7 +39,7 @@ class KafkaWrapper(object):
     def Kafka_DeleteTopic(self, p_szTopicName):
         a = AdminClient({'bootstrap.servers': self.__kafka_servers__})
         deleted_topics = [p_szTopicName, ]
-        fs = a.delete_topics(deleted_topics)
+        fs = a.delete_topics(topics=deleted_topics)
         for topic, f in fs.items():
             try:
                 f.result()
@@ -186,11 +186,14 @@ class KafkaWrapper(object):
                             m_szSQL, re.IGNORECASE | re.DOTALL)
         if matchObj:
             m_TopicName = str(matchObj.group(1)).strip()
-            m_formula_str = str(matchObj.group(2)).strip()
-            m_row_struct = parse_formula_str(m_formula_str)
+            m_RawMessages = str(matchObj.group(2)).split('\n')
+            m_Messages = []
+            for m_nPos in range(0, len(m_RawMessages)):
+                if len(m_RawMessages[m_nPos]) != 0:
+                    m_Messages.append(get_final_string(parse_formula_str(m_RawMessages[m_nPos])))
             m_ProduceError = []
             try:
-                nTotalCount = self.kafka_Produce(m_TopicName, get_final_string(m_row_struct), m_ProduceError)
+                nTotalCount = self.kafka_Produce(m_TopicName, m_Messages, m_ProduceError)
                 if len(m_ProduceError) != 0:
                     return None, None, None, "Total {} messages send to topic {} with {} failed.".\
                         format(nTotalCount, m_TopicName, len(m_ProduceError))
@@ -204,7 +207,7 @@ class KafkaWrapper(object):
                             m_szSQL, re.IGNORECASE | re.DOTALL)
         if matchObj:
             m_TopicName = str(matchObj.group(1)).strip()
-            m_formula_str = str(matchObj.group(2)).strip()
+            m_formula_str = str(matchObj.group(2)).replace('\r', '').replace('\n', '').strip()
             m_row_struct = parse_formula_str(m_formula_str)
             m_row_count = int(str(matchObj.group(4)).strip())
             m_ErrorCount = 0
@@ -232,7 +235,7 @@ class KafkaWrapper(object):
             except (KafkaException, KafkaWrapperException) as ke:
                 return None, None, None, "Failed to send message for topic {}: {}".format(m_TopicName, repr(ke))
 
-        print("Not Match anything")
+        return None, None, None, "Unknown kafka Command."
 
 
 if __name__ == '__main__':
